#!/usr/bin/perl -w

use strict;
use Net::STOMP::Client;
use JSON qw( decode_json encode_json );

my $login = 'sys-webcluster';
my $password = '********************',
my $vhost = 'idm';

my $dienst = "text";
my $queue = "sys-webserver-2";

my $stomp ;


# vom Dienst selbst zu implementierten, erhält als Parameter eine JSON-Struktur
# muss ggf. selbst über "sendAuditMessage" Nachrichten generieren
# im Erfolgsfall Rückgabewert true (z.B. 1), sonst false
sub handleMessage($) {
	print "handleMessage\n";
	my ($payload) = @_;
	print $payload->{accountId} . "\n";
        print $payload->{type} . "\n";
        sendAuditMessage($payload->{accountId}, "alles gescheitert", 0);
	return 0;
}

sub sendAuditMessage($$$) {
	print "sendingAuditMessage\n";
	my ($accountId, $message, $status)  = @_;
	 my $auditData = {};
    $auditData->{'mqLogin'} = $login;
    $auditData->{'vhost'} = $vhost;
    $auditData->{'dienst'} = $dienst;
    $auditData->{'queue'} = $queue;
    $auditData->{'accountId'} = $accountId;
    $auditData->{'message'} = $message;
    $auditData->{'status'} = $status;
    $auditData->{'auditVersion'} = "0.1";
    $auditData->{'zeitStempel'} = time();
    
    $stomp->send(
        destination => '/exchange/audit',
        subject     => 'Audit-Nachricht',
        time        => time(),
        body        => encode_json($auditData),
        persistent =>
          'true' # wichtig, sonst werden Nachrichten beim Serverneustart gelöscht
    );
	
}

$stomp =
  Net::STOMP::Client->new( uri => "stomp+ssl://scc-mq-01.scc.kit.edu:61614" );

$stomp->message_callback(
    sub {
        my ( $self, $frame ) = @_;
        print "this is the callback\n";
        my $payload = decode_json( $frame->body );
        if (handleMessage($payload)) {
        $self->ack( frame => $frame )
          ;  # explizites Ack, sonst wird die Nachricht später neu ausgeliefert
        } else {
        $self->nack( frame => $frame )          ;  # explizites NACK
        	
        return;}
    }
);

my $peer = $stomp->peer();
printf( "connected to broker %s (IP %s), port %d\n",
    $peer->host(), $peer->addr(), $peer->port() );
$stomp->connect(
    login    => $login,
    passcode => $password,
    host     => $vhost
);
print "connected\n";

my $subscriptionId = $stomp->uuid();
printf( "speaking STOMP %s with server %s\n",
    $stomp->version(), $stomp->server() || "UNKNOWN" );
printf( "session %s started\n", $stomp->session() );

# der Parameter "ack=>'client'" sorgt dafür, dass nicht jede ausgelieferte Message automatisch quittiert wird.
$stomp->subscribe(
    destination => "/queue/$queue",
    id          => $subscriptionId,
    receipt     => $stomp->uuid(),
    ack         => 'client'
);
print "subscription completed\n";

# Ein Daemon geht natürlich anders schöner, aber hier geht es nur um das lesen aus der Queue.
my $count = 0;
while ( $count < 100 ) {
    my $frame = $stomp->wait_for_frames( timeout => 1 );
    if ($frame) {
        $count++;
    }
    else {
        print("waiting for messages...\n");
    }
}

$stomp->disconnect();
printf("session ended\n");

